package com.twitter.finagle.service

import com.twitter.util.Future

import com.twitter.finagle.{Service, ServiceClosedException}
import com.twitter.finagle.util.DrainablePool

/**
 * A pool of services.
 */
class PoolingService[Req, Rep](pool: DrainablePool[Service[Req, Rep]])
  extends Service[Req, Rep]
{
  private[this] var isOpen = true

  def apply(request: Req): Future[Rep] = synchronized {
    if (!isOpen) return Future.exception(new ServiceClosedException)

    pool.reserve() flatMap { service =>
      service(request) ensure { pool.release(service) }
    }
  }

  // TODO: include measures of underlying health / pool occupancy?
  override def isAvailable = synchronized { isOpen }

  override def close() = synchronized {
    isOpen = false
    pool.drain()
  }
}

